@@ -218,12 +218,27 @@ const (
 	// PoolInfoAction - allow pool specific summary and detail information
 	PoolInfoAction = "admin:PoolInfo"
 
+	// NodeListAction - allow listing of nodes
+	NodeListAction = "admin:NodeList"
+
+	// NodeInfoAction - allow node specific summary and detailed information
+	NodeInfoAction = "admin:NodeInfo"
+
 	// SetInfoAction - allow set specific summary and detail
 	SetInfoAction = "admin:SetInfo"
 
+	// DriveListAction - allow listing of drives
+	DriveListAction = "admin:DriveList"
+
 	// DriveInfoAction - allow drive specific summary and detail
 	DriveInfoAction = "admin:DriveInfo"
 
+	//  SetQOSConfigAction - allow set QOS configuration
+	SetQOSConfigAction = "admin:SetQOSConfig"
+
+	//  GetQOSConfigAction - allow get QOS configuration
+	GetQOSConfigAction = "admin:GetQOSConfig"
+
 	// AllAdminActions - provides all admin permissions
 	AllAdminActions = "admin:*"
 )
@@ -242,7 +257,9 @@ var supportedAdminActions = map[AdminAction]struct{}{
 	KMSKeyStatusAdminAction:          {},
 	ServerInfoAdminAction:            {},
 	HealthInfoAdminAction:            {},
+	LicenseInfoAdminAction:           {},
 	BandwidthMonitorAction:           {},
+	InspectDataAction:                {},
 	ServerUpdateAdminAction:          {},
 	ServiceRestartAdminAction:        {},
 	ServiceStopAdminAction:           {},
@@ -297,6 +314,17 @@ var supportedAdminActions = map[AdminAction]struct{}{
 	StartBatchJobAction:    {},
 	CancelBatchJobAction:   {},
 
+	ClusterInfoAction:  {},
+	PoolListAction:     {},
+	PoolInfoAction:     {},
+	NodeListAction:     {},
+	NodeInfoAction:     {},
+	SetInfoAction:      {},
+	DriveListAction:    {},
+	DriveInfoAction:    {},
+	SetQOSConfigAction: {},
+	GetQOSConfigAction: {},
+
 	AllAdminActions: {},
 }
 
