@@ -157,6 +157,9 @@ const (
 	// S3AuthType - optionally use this condition key to restrict incoming requests to use a specific authentication method.
 	S3AuthType KeyName = "s3:authType"
 
+	// S3ExpressSessionMode - optionally use this condition key to control who can create a ReadWrite or ReadOnly session.
+	S3ExpressSessionMode = "s3express:SessionMode"
+
 	// Refer https://docs.aws.amazon.com/AmazonS3/latest/userguide/tagging-and-policies.html
 	ExistingObjectTag    KeyName = "s3:ExistingObjectTag"
 	RequestObjectTagKeys KeyName = "s3:RequestObjectTagKeys"
@@ -212,6 +215,7 @@ const (
 const (
 	// STSDurationSeconds - Duration seconds condition for STS policy
 	STSDurationSeconds KeyName = "sts:DurationSeconds"
+
 	// SVCDurationSeconds - Duration seconds condition for Admin policy
 	SVCDurationSeconds KeyName = "svc:DurationSeconds"
 )
@@ -313,6 +317,7 @@ var CommonKeys = append([]KeyName{
 	S3SignatureAge,
 	S3XAmzContentSha256,
 	S3LocationConstraint,
+	S3ExpressSessionMode,
 	AWSReferer,
 	AWSSourceIP,
 	AWSUserAgent,
@@ -359,6 +364,14 @@ var AllSupportedAdminKeys = append([]KeyName{
 
 // AllSupportedSTSKeys is the all supported conditions for STS policies
 var AllSupportedSTSKeys = []KeyName{
+	AWSPrincipalType,
+	AWSSecureTransport,
 	STSDurationSeconds,
+	LDAPUser,
+	AWSUserID,
+	AWSGroups,
+	LDAPGroups,
+	LDAPUsername,
+	AWSUsername,
 	// Add new supported condition keys.
 }
